@@ -23,6 +23,11 @@ module Language.Haskell.Exts.Pretty (
                 PPHsMode(..), Indent, PPLayout(..), defaultMode) where
 
 import Language.Haskell.Exts.Syntax
+import qualified Language.Haskell.Exts.Annotated.Syntax as A
+import Language.Haskell.Exts.Annotated.Simplify
+import qualified Language.Haskell.Exts.ParseSyntax as P
+
+import Language.Haskell.Exts.SrcLoc
 
 import qualified Text.PrettyPrint as P
 import Data.List (intersperse)
@@ -424,8 +429,10 @@ instance Pretty Decl where
                 mySep ((punctuate comma . map pretty $ nameList)
                       ++ [text "::", pretty qualType])
 
-        pretty (FunBind matches) =
-                foldr ($$$) empty (map pretty matches)
+        pretty (FunBind matches) = do
+                e <- fmap layout getPPEnv
+                case e of PPOffsideRule -> foldr ($$$) empty (map pretty matches)
+                          _ -> foldr (\x y -> x <> semi <> y) empty (map pretty matches)
 
         pretty (PatBind pos pat optsig rhs whereBinds) =
                 markLine pos $
@@ -705,9 +712,11 @@ ppForall (Just vs) =    myFsep (text "forall" : map pretty vs ++ [char '.'])
 ---------------------------- Kinds ----------------------------
 
 instance Pretty Kind where
-        pretty KindStar     = text "*"
-        pretty KindBang     = text "!"
-        pretty (KindFn a b) = myFsep [pretty a, text "->", pretty b]
+        pretty KindStar      = text "*"
+        pretty KindBang      = text "!"
+        pretty (KindFn a b)  = myFsep [pretty a, text "->", pretty b]
+        pretty (KindParen k) = parens $ pretty k
+        pretty (KindVar n)   = pretty n
 
 ppOptKind :: Maybe Kind -> [Doc]
 ppOptKind Nothing  = []
@@ -776,6 +785,7 @@ instance Pretty Exp where
         pretty (IPVar ipname) = pretty ipname
         pretty (Con name) = pretty name
         pretty (Tuple expList) = parenList . map pretty $ expList
+        pretty (TupleSection mExpList) = parenList . map (maybePP pretty) $ mExpList
         -- weird stuff
         pretty (Paren e) = parens . pretty $ e
         pretty (LeftSection e op) = parens (pretty e <+> pretty op)
@@ -829,8 +839,6 @@ instance Pretty Exp where
                 myFsep $ [text "{-# GENERATED", text $ show s,
                             int a, char ':', int b, char '-',
                             int c, char ':', int d, text "#-}", pretty e]
-{-        pretty (UnknownExpPragma n s) =
-                myFsep $ [text "{-#", text n, text s, text "#-}"] -}
         -- Arrows
         pretty (Proc p e) = myFsep $ [text "proc", pretty p, text "->", pretty e]
         pretty (LeftArrApp l r)      = myFsep $ [pretty l, text "-<",  pretty r]
@@ -1057,6 +1065,9 @@ instance Pretty CName where
         pretty (VarName n) = pretty n
         pretty (ConName n) = pretty n
 
+instance Pretty SpecialCon where
+	pretty sc = text $ specialName sc
+
 isSymbolName :: Name -> Bool
 isSymbolName (Symbol _) = True
 isSymbolName _ = False
@@ -1087,6 +1098,299 @@ instance Pretty Asst where
         pretty (IParam i t)    = myFsep $ [pretty i, text "::", pretty t]
         pretty (EqualP t1 t2)  = myFsep $ [pretty t1, text "~", pretty t2]
 
+-- Pretty print a source location, useful for printing out error messages
+instance Pretty SrcLoc where
+  pretty srcLoc =
+    return $ P.hsep [ colonFollow (P.text $ srcFilename srcLoc)
+                    , colonFollow (P.int  $ srcLine     srcLoc)
+                    , P.int $ srcColumn srcLoc
+                    ]
+
+colonFollow p = P.hcat [ p, P.colon ]
+
+
+instance Pretty SrcSpan where
+    pretty srcSpan =
+        return $ P.hsep [ colonFollow (P.text $ srcSpanFilename srcSpan)
+                        , P.hcat [ P.text "("
+                                 , P.int $ srcSpanStartLine srcSpan
+                                 , P.colon
+                                 , P.int $ srcSpanStartColumn srcSpan
+                                 , P.text ")"
+                                 ]
+                        , P.text "-"
+                        , P.hcat [ P.text "("
+                                 , P.int $ srcSpanEndLine srcSpan
+                                 , P.colon
+                                 , P.int $ srcSpanEndColumn srcSpan
+                                 , P.text ")"
+                                 ]
+                        ]
+
+---------------------------------------------------------------------
+-- Annotated version
+
+-------------------------  Pretty-Print a Module --------------------
+instance SrcInfo pos => Pretty (A.Module pos) where
+        pretty (A.Module pos mbHead os imp decls) =
+                markLine pos $
+                myVcat $ map pretty os ++
+                    (case mbHead of
+                        Nothing -> id
+                        Just h  -> \x -> [topLevel (pretty h) x])
+                    (map pretty imp ++ map pretty decls)
+        pretty (A.XmlPage pos _mn os n attrs mattr cs) =
+                markLine pos $
+                myVcat $ map pretty os ++
+                    [let ax = maybe [] (return . pretty) mattr
+                      in hcat $
+                         (myFsep $ (char '<' <> pretty n): map pretty attrs ++ ax ++ [char '>']):
+                            map pretty cs ++ [myFsep $ [text "</" <> pretty n, char '>']]]
+        pretty (A.XmlHybrid pos mbHead os imp decls n attrs mattr cs) =
+                markLine pos $
+                myVcat $ map pretty os ++ [text "<%"] ++
+                    (case mbHead of
+                        Nothing -> id
+                        Just h  -> \x -> [topLevel (pretty h) x])
+                    (map pretty imp ++ map pretty decls ++
+                        [let ax = maybe [] (return . pretty) mattr
+                          in hcat $
+                             (myFsep $ (char '<' <> pretty n): map pretty attrs ++ ax ++ [char '>']):
+                                map pretty cs ++ [myFsep $ [text "</" <> pretty n, char '>']]])
+
+
+
+--------------------------  Module Header ------------------------------
+instance Pretty (A.ModuleHead l) where
+    pretty (A.ModuleHead _ m mbWarn mbExportList) = mySep [
+        text "module",
+        pretty m,
+        maybePP pretty mbWarn,
+        maybePP pretty mbExportList,
+        text "where"]
+
+instance Pretty (A.WarningText l) where
+    pretty = ppWarnTxt. sWarningText
+
+instance Pretty (A.ModuleName l) where
+        pretty = pretty . sModuleName
+
+instance Pretty (A.ExportSpecList l) where
+        pretty (A.ExportSpecList _ especs)  = parenList $ map pretty especs
+
+instance Pretty (A.ExportSpec l) where
+        pretty = pretty . sExportSpec
+
+instance SrcInfo pos => Pretty (A.ImportDecl pos) where
+        pretty = pretty . sImportDecl
+
+instance Pretty (A.ImportSpecList l) where
+        pretty (A.ImportSpecList _ b ispecs)  =
+            (if b then text "hiding" else empty)
+                <+> parenList (map pretty ispecs)
+
+instance Pretty (A.ImportSpec l) where
+        pretty = pretty . sImportSpec
+
+-------------------------  Declarations ------------------------------
+instance SrcInfo pos => Pretty (A.Decl pos) where
+        pretty = pretty . sDecl
+
+instance Pretty (A.DeclHead l) where
+    pretty (A.DHead l n tvs)       = mySep (pretty n : map pretty tvs)
+    pretty (A.DHInfix l tva n tvb) = mySep [pretty tva, pretty n, pretty tvb]
+    pretty (A.DHParen l dh)        = parens (pretty dh)
+
+instance Pretty (A.InstHead l) where
+    pretty (A.IHead l qn ts)       = mySep (pretty qn : map pretty ts)
+    pretty (A.IHInfix l ta qn tb)  = mySep [pretty ta, pretty qn, pretty tb]
+    pretty (A.IHParen l ih)        = parens (pretty ih)
+
+instance Pretty (A.DataOrNew l) where
+        pretty = pretty . sDataOrNew
+
+instance Pretty (A.Assoc l) where
+        pretty = pretty . sAssoc
+
+instance SrcInfo pos => Pretty (A.Match pos) where
+        pretty = pretty . sMatch
+
+instance SrcInfo loc => Pretty (A.ClassDecl loc) where
+        pretty = pretty . sClassDecl
+
+instance SrcInfo loc => Pretty (A.InstDecl loc) where
+        pretty = pretty . sInstDecl
+
+------------------------- FFI stuff -------------------------------------
+instance Pretty (A.Safety l) where
+        pretty = pretty . sSafety
+
+instance Pretty (A.CallConv l) where
+        pretty = pretty . sCallConv
+
+------------------------- Pragmas ---------------------------------------
+instance SrcInfo loc => Pretty (A.Rule loc) where
+        pretty = pretty . sRule
+
+instance Pretty (A.Activation l) where
+    pretty = pretty . sActivation
+
+instance Pretty (A.RuleVar l) where
+    pretty = pretty . sRuleVar
+
+instance Pretty (A.OptionPragma l) where
+    pretty (A.LanguagePragma _ ns) =
+        myFsep $ text "{-# LANGUAGE" : punctuate (char ',') (map pretty ns) ++ [text "#-}"]
+    pretty (A.IncludePragma _ s) =
+        myFsep $ [text "{-# INCLUDE", text s, text "#-}"]
+    pretty (A.CFilesPragma _ s) =
+        myFsep $ [text "{-# CFILES", text s, text "#-}"]
+    pretty (A.OptionsPragma _ (Just tool) s) =
+        myFsep $ [text "{-# OPTIONS_" <> pretty tool, text s, text "#-}"]
+    pretty (A.OptionsPragma _ _ s) =
+        myFsep $ [text "{-# OPTIONS", text s, text "#-}"]
+
+------------------------- Data & Newtype Bodies -------------------------
+instance Pretty (A.QualConDecl l) where
+        pretty (A.QualConDecl _pos mtvs ctxt con) =
+                myFsep [ppForall (fmap (map sTyVarBind) mtvs), ppContext $ maybe [] sContext ctxt, pretty con]
+
+instance Pretty (A.GadtDecl l) where
+        pretty (A.GadtDecl _pos name ty) =
+                myFsep [pretty name, text "::", pretty ty]
+
+instance Pretty (A.ConDecl l) where
+        pretty = pretty . sConDecl
+
+instance Pretty (A.FieldDecl l) where
+        pretty (A.FieldDecl _ names ty) =
+                myFsepSimple $ (punctuate comma . map pretty $ names) ++
+                       [text "::", pretty ty]
+
+
+instance Pretty (A.BangType l) where
+        pretty = pretty . sBangType
+
+instance Pretty (A.Deriving l) where
+        pretty (A.Deriving _ []) = text "deriving" <+> parenList []
+        pretty (A.Deriving _ [A.IHead _ d []]) = text "deriving" <+> pretty d
+        pretty (A.Deriving _ ihs) = text "deriving" <+> parenList (map pretty ihs)
+
+------------------------- Types -------------------------
+instance Pretty (A.Type l) where
+        pretty = pretty . sType
+
+instance Pretty (A.TyVarBind l) where
+        pretty = pretty . sTyVarBind
+
+---------------------------- Kinds ----------------------------
+
+instance Pretty (A.Kind l) where
+        pretty = pretty . sKind
+
+------------------- Functional Dependencies -------------------
+instance Pretty (A.FunDep l) where
+        pretty = pretty . sFunDep
+
+------------------------- Expressions -------------------------
+instance SrcInfo loc => Pretty (A.Rhs loc) where
+        pretty = pretty . sRhs
+
+instance SrcInfo loc => Pretty (A.GuardedRhs loc) where
+        pretty = pretty . sGuardedRhs
+
+instance Pretty (A.Literal l) where
+        pretty = pretty . sLiteral
+
+instance SrcInfo loc => Pretty (A.Exp loc) where
+        pretty = pretty . sExp
+
+instance SrcInfo loc => Pretty (A.XAttr loc) where
+        pretty = pretty . sXAttr
+
+instance Pretty (A.XName l) where
+        pretty = pretty . sXName
+
+--------------------- Template Haskell -------------------------
+
+instance SrcInfo loc => Pretty (A.Bracket loc) where
+        pretty = pretty . sBracket
+
+instance SrcInfo loc => Pretty (A.Splice loc) where
+        pretty = pretty . sSplice
+
+------------------------- Patterns -----------------------------
+
+instance SrcInfo loc => Pretty (A.Pat loc) where
+        pretty = pretty . sPat
+
+instance SrcInfo loc => Pretty (A.PXAttr loc) where
+        pretty = pretty . sPXAttr
+
+instance SrcInfo loc => Pretty (A.PatField loc) where
+        pretty = pretty . sPatField
+
+--------------------- Regular Patterns -------------------------
+
+instance SrcInfo loc => Pretty (A.RPat loc) where
+        pretty = pretty . sRPat
+
+instance Pretty (A.RPatOp l) where
+        pretty = pretty . sRPatOp
+
+------------------------- Case bodies  -------------------------
+instance SrcInfo loc => Pretty (A.Alt loc) where
+        pretty = pretty . sAlt
+
+instance SrcInfo loc => Pretty (A.GuardedAlts loc) where
+        pretty = pretty . sGuardedAlts
+
+instance SrcInfo loc => Pretty (A.GuardedAlt loc) where
+        pretty = pretty . sGuardedAlt
+
+------------------------- Statements in monads, guards & list comprehensions -----
+instance SrcInfo loc => Pretty (A.Stmt loc) where
+        pretty = pretty . sStmt
+
+instance SrcInfo loc => Pretty (A.QualStmt loc) where
+        pretty = pretty . sQualStmt
+
+------------------------- Record updates
+instance SrcInfo loc => Pretty (A.FieldUpdate loc) where
+        pretty = pretty . sFieldUpdate
+
+------------------------- Names -------------------------
+instance Pretty (A.QOp l) where
+        pretty = pretty . sQOp
+
+instance Pretty (A.QName l) where
+        pretty = pretty . sQName
+
+instance Pretty (A.Op l) where
+        pretty = pretty . sOp
+
+instance Pretty (A.Name l) where
+        pretty = pretty . sName
+
+instance Pretty (A.IPName l) where
+        pretty = pretty . sIPName
+
+instance SrcInfo loc => Pretty (A.IPBind loc) where
+        pretty = pretty . sIPBind
+
+instance Pretty (A.CName l) where
+        pretty = pretty . sCName
+
+instance Pretty (A.Context l) where
+        pretty (A.CxEmpty _) = mySep [text "()", text "=>"]
+        pretty (A.CxSingle _ asst) = mySep [pretty asst, text "=>"]
+        pretty (A.CxTuple _ assts) = myFsep $ [parenList (map pretty assts), text "=>"]
+        pretty (A.CxParen _ asst)  = parens (pretty asst)
+
+-- hacked for multi-parameter type classes
+instance Pretty (A.Asst l) where
+        pretty = pretty . sAsst
+
 ------------------------- pp utils -------------------------
 maybePP :: (a -> Doc) -> Maybe a -> Doc
 maybePP pp Nothing = empty
@@ -1178,21 +1482,156 @@ layoutChoice a b dl = do e <- getPPEnv
 -- that the following line is line n.  But if there's no newline before
 -- the line we're talking about, we need to compensate by adding 1.
 
-markLine :: SrcLoc -> Doc -> Doc
+markLine :: SrcInfo s => s -> Doc -> Doc
 markLine loc doc = do
         e <- getPPEnv
-        let y = srcLine loc
+        let y = startLine loc
         let line l =
-              text ("{-# LINE " ++ show l ++ " \"" ++ srcFilename loc ++ "\" #-}")
+              text ("{-# LINE " ++ show l ++ " \"" ++ fileName loc ++ "\" #-}")
         if linePragmas e then layoutChoice (line y $$) (line (y+1) <+>) doc
               else doc
 
--- Pretty print a source location, useful for printing out error messages
-instance Pretty SrcLoc where
-  pretty srcLoc =
-    return $ P.hsep [ colonFollow (P.text $ srcFilename srcLoc)
-                    , colonFollow (P.int  $ srcLine     srcLoc)
-                    , P.int $ srcColumn srcLoc
-                    ]
-    where
-    colonFollow p = P.hcat [ p, P.colon ]
+--------------------------------------------------------------------------------
+-- Pretty-printing of internal constructs, for error messages while parsing
+
+instance SrcInfo loc => Pretty (P.PExp loc) where
+        pretty (P.Lit _ l) = pretty l
+        pretty (P.InfixApp _ a op b) = myFsep [pretty a, pretty op, pretty b]
+        pretty (P.NegApp _ e) = myFsep [char '-', pretty e]
+        pretty (P.App _ a b) = myFsep [pretty a, pretty b]
+        pretty (P.Lambda _loc expList ppBody) = myFsep $
+                char '\\' : map pretty expList ++ [text "->", pretty ppBody]
+        pretty (P.Let _ (A.BDecls _ declList) letBody) =
+                ppLetExp declList letBody
+        pretty (P.Let _ (A.IPBinds _ bindList) letBody) =
+                ppLetExp bindList letBody
+        pretty (P.If _ cond thenexp elsexp) =
+                myFsep [text "if", pretty cond,
+                        text "then", pretty thenexp,
+                        text "else", pretty elsexp]
+        pretty (P.Case _ cond altList) =
+                myFsep [text "case", pretty cond, text "of"]
+                $$$ ppBody caseIndent (map pretty altList)
+        pretty (P.Do _ stmtList) =
+                text "do" $$$ ppBody doIndent (map pretty stmtList)
+        pretty (P.MDo _ stmtList) =
+                text "mdo" $$$ ppBody doIndent (map pretty stmtList)
+        pretty (P.Var _ name) = pretty name
+        pretty (P.IPVar _ ipname) = pretty ipname
+        pretty (P.Con _ name) = pretty name
+        pretty (P.TupleSection _ mExpList) = parenList . map (maybePP pretty) $ mExpList
+        pretty (P.Paren _ e) = parens . pretty $ e
+        pretty (P.RecConstr _ c fieldList) =
+                pretty c <> (braceList . map pretty $ fieldList)
+        pretty (P.RecUpdate _ e fieldList) =
+                pretty e <> (braceList . map pretty $ fieldList)
+        pretty (P.List _ list) =
+                bracketList . punctuate comma . map pretty $ list
+        pretty (P.EnumFrom _ e) =
+                bracketList [pretty e, text ".."]
+        pretty (P.EnumFromTo _ from to) =
+                bracketList [pretty from, text "..", pretty to]
+        pretty (P.EnumFromThen _ from thenE) =
+                bracketList [pretty from <> comma, pretty thenE, text ".."]
+        pretty (P.EnumFromThenTo _ from thenE to) =
+                bracketList [pretty from <> comma, pretty thenE,
+                             text "..", pretty to]
+        pretty (P.ParComp _ e qualLists) =
+                bracketList (intersperse (char '|') $
+                                pretty e : (punctuate comma . concatMap (map pretty) $ qualLists))
+        pretty (P.ExpTypeSig _pos e ty) =
+                myFsep [pretty e, text "::", pretty ty]
+        pretty (P.BracketExp _ b) = pretty b
+        pretty (P.SpliceExp _ s) = pretty s
+        pretty (P.TypQuote _ t)  = text "\'\'" <> pretty t
+        pretty (P.VarQuote _ x)  = text "\'" <> pretty x
+        pretty (P.QuasiQuote _ n qt) = text ("[$" ++ n ++ "|" ++ qt ++ "|]")
+        pretty (P.XTag _ n attrs mattr cs) =
+                let ax = maybe [] (return . pretty) mattr
+                 in hcat $
+                     (myFsep $ (char '<' <> pretty n): map pretty attrs ++ ax ++ [char '>']):
+                        map pretty cs ++ [myFsep $ [text "</" <> pretty n, char '>']]
+        pretty (P.XETag _ n attrs mattr) =
+                let ax = maybe [] (return . pretty) mattr
+                 in myFsep $ (char '<' <> pretty n): map pretty attrs ++ ax ++ [text "/>"]
+        pretty (P.XPcdata _ s) = text s
+        pretty (P.XExpTag _ e) =
+                myFsep $ [text "<%", pretty e, text "%>"]
+        pretty (P.CorePragma _ s e) = myFsep $ map text ["{-# CORE", show s, "#-}"] ++ [pretty e]
+        pretty (P.SCCPragma  _ s e) = myFsep $ map text ["{-# SCC",  show s, "#-}"] ++ [pretty e]
+        pretty (P.GenPragma  _ s (a,b) (c,d) e) =
+                myFsep $ [text "{-# GENERATED", text $ show s,
+                            int a, char ':', int b, char '-',
+                            int c, char ':', int d, text "#-}", pretty e]
+        pretty (P.Proc _ p e) = myFsep $ [text "proc", pretty p, text "->", pretty e]
+        pretty (P.LeftArrApp _ l r)      = myFsep $ [pretty l, text "-<",  pretty r]
+        pretty (P.RightArrApp _ l r)     = myFsep $ [pretty l, text ">-",  pretty r]
+        pretty (P.LeftArrHighApp _ l r)  = myFsep $ [pretty l, text "-<<", pretty r]
+        pretty (P.RightArrHighApp _ l r) = myFsep $ [pretty l, text ">>-", pretty r]
+        pretty (P.AsPat _ name (P.IrrPat _ pat)) =
+                myFsep [pretty name <> char '@', char '~' <> pretty pat]
+        pretty (P.AsPat _ name pat) =
+                hcat [pretty name, char '@', pretty pat]
+        pretty (P.WildCard _) = char '_'
+        pretty (P.IrrPat _ pat) = char '~' <> pretty pat
+        pretty (P.PostOp _ e op) = pretty e <+> pretty op
+        pretty (P.PreOp _ op e)  = pretty op <+> pretty e
+        pretty (P.ViewPat _ e p) =
+                myFsep [pretty e, text "->", pretty p]
+        pretty (P.SeqRP _ rs) = myFsep $ text "(/" : map pretty rs ++ [text "/)"]
+        pretty (P.GuardRP _ r gs) =
+                myFsep $ text "(|" : pretty r : char '|' : map pretty gs ++ [text "|)"]
+        pretty (P.EitherRP _ r1 r2) = parens . myFsep $ [pretty r1, char '|', pretty r2]
+        pretty (P.CAsRP _ n (P.IrrPat _ e)) =
+                myFsep [pretty n <> text "@:", char '~' <> pretty e]
+        pretty (P.CAsRP _ n r) = hcat [pretty n, text "@:", pretty r]
+        pretty (P.XRPats _ ps) =
+                myFsep $ text "<[" : map pretty ps ++ [text "%>"]
+        pretty (P.ExplTypeArg _ qn t) =
+                myFsep [pretty qn, text "{|", pretty t, text "|}"]
+        pretty (P.BangPat _ e) = text "!" <> pretty e
+
+instance SrcInfo loc => Pretty (P.PFieldUpdate loc) where
+        pretty (P.FieldUpdate _ name e) =
+                myFsep [pretty name, equals, pretty e]
+        pretty (P.FieldPun _ name) = pretty name
+        pretty (P.FieldWildcard _) = text ".."
+
+instance SrcInfo loc => Pretty (P.ParseXAttr loc) where
+        pretty (P.XAttr _ n v) =
+                myFsep [pretty n, char '=', pretty v]
+
+instance SrcInfo loc => Pretty (P.PContext loc) where
+        pretty (P.CxEmpty _) = mySep [text "()", text "=>"]
+        pretty (P.CxSingle _ asst) = mySep [pretty asst, text "=>"]
+        pretty (P.CxTuple _ assts) = myFsep $ [parenList (map pretty assts), text "=>"]
+        pretty (P.CxParen _ asst)  = parens (pretty asst)
+
+instance SrcInfo loc => Pretty (P.PAsst loc) where
+        pretty (P.ClassA _ a ts)   = myFsep $ ppQName (sQName a) : map (prettyPrec prec_atype) ts
+        pretty (P.InfixA _ a op b) = myFsep $ [pretty a, ppQNameInfix (sQName op), pretty b]
+        pretty (P.IParam _ i t)    = myFsep $ [pretty i, text "::", pretty t]
+        pretty (P.EqualP _ t1 t2)  = myFsep $ [pretty t1, text "~", pretty t2]
+
+instance SrcInfo loc => Pretty (P.PType loc) where
+        prettyPrec p (P.TyForall _ mtvs ctxt htype) = parensIf (p > 0) $
+                myFsep [ppForall (fmap (map sTyVarBind) mtvs), maybePP pretty ctxt, pretty htype]
+        prettyPrec p (P.TyFun _ a b) = parensIf (p > 0) $
+                myFsep [prettyPrec prec_btype a, text "->", pretty b]
+        prettyPrec _ (P.TyTuple _ bxd l) =
+                let ds = map pretty l
+                 in case bxd of
+                        Boxed   -> parenList ds
+                        Unboxed -> hashParenList ds
+        prettyPrec _ (P.TyList _ t)  = brackets $ pretty t
+        prettyPrec p (P.TyApp _ a b) =
+                {-
+                | a == list_tycon = brackets $ pretty b         -- special case
+                | otherwise = -} parensIf (p > prec_btype) $
+                                    myFsep [pretty a, prettyPrec prec_atype b]
+        prettyPrec _ (P.TyVar _ name) = pretty name
+        prettyPrec _ (P.TyCon _ name) = pretty name
+        prettyPrec _ (P.TyParen _ t) = parens (pretty t)
+        prettyPrec _ (P.TyPred _ asst) = pretty asst
+        prettyPrec _ (P.TyInfix _ a op b) = myFsep [pretty a, ppQNameInfix (sQName op), pretty b]
+        prettyPrec _ (P.TyKind _ t k) = parens (myFsep [pretty t, text "::", pretty k])
