@@ -103,7 +103,11 @@ import Data.Generics (Data(..),Typeable(..))
 #endif
 #endif
 
--- | A position in the source.
+import Language.Haskell.Exts.SrcLoc (SrcLoc(..))
+
+import Language.Haskell.Exts.Annotated.Syntax (Boxed(..), Tool(..))
+
+{-- | A position in the source.
 data SrcLoc = SrcLoc {
         srcFilename :: String,
         srcLine :: Int,
@@ -114,6 +118,7 @@ data SrcLoc = SrcLoc {
 #else
   deriving (Eq,Ord,Show)
 #endif
+-}
 
 -- | The name of a Haskell module.
 newtype ModuleName = ModuleName String
@@ -208,9 +213,9 @@ data CName
 data Module = Module SrcLoc ModuleName [OptionPragma] (Maybe WarningText)
                         (Maybe [ExportSpec]) [ImportDecl] [Decl]
 #ifdef __GLASGOW_HASKELL__
-  deriving (Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An item in a module's export specification.
@@ -228,9 +233,9 @@ data ExportSpec
      | EModuleContents ModuleName     -- ^ @module M@:
                                       -- re-export a module.
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An import declaration.
@@ -247,9 +252,9 @@ data ImportDecl = ImportDecl
             -- by @hiding@.
     }
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An import specification, representing a single explicit item imported
@@ -265,9 +270,9 @@ data ImportSpec
                                 -- a class imported with some of its methods, or
                                 -- a datatype imported with some of its constructors.
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Associativity of an operator.
@@ -276,9 +281,9 @@ data Assoc
      | AssocLeft  -- ^ left-associative operator (declared with @infixl@).
      | AssocRight -- ^ right-associative operator (declared with @infixr@)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A single derived instance, which may have arguments since it may be a MPTC.
@@ -312,7 +317,7 @@ data Decl
      -- ^ A declaration of operator fixity
      | DefaultDecl  SrcLoc [Type]
      -- ^ A declaration of default types
-     | SpliceDecl   SrcLoc Splice
+     | SpliceDecl   SrcLoc Exp
      -- ^ A Template Haskell splicing declaration
      | TypeSig      SrcLoc [Name] Type
      -- ^ A type signature declaration
@@ -340,17 +345,17 @@ data Decl
      | InstSig          SrcLoc Context         QName [Type]
      -- ^ A SPECIALISE instance pragma
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A flag stating whether a declaration is a data or newtype declaration.
 data DataOrNew = DataType | NewType
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A binding group inside a @let@ or @where@ clause.
@@ -358,26 +363,26 @@ data Binds
     = BDecls [Decl]     -- ^ An ordinary binding group
     | IPBinds [IPBind]  -- ^ A binding group for implicit parameters
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A binding of an implicit parameter.
 data IPBind = IPBind SrcLoc IPName Exp
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Clauses of a function binding.
 data Match
      = Match SrcLoc Name [Pat] (Maybe Type) Rhs {-where-} Binds
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A single constructor declaration within a data type declaration,
@@ -387,9 +392,9 @@ data QualConDecl
         {-forall-} [TyVarBind] {- . -} Context
         {- => -} ConDecl
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Declaration of an ordinary data constructor.
@@ -401,18 +406,18 @@ data ConDecl
      | RecDecl Name [([Name],BangType)]
                 -- ^ record constructor
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A single constructor declaration in a GADT data type declaration.
 data GadtDecl
     = GadtDecl SrcLoc Name Type
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Declarations inside a class declaration.
@@ -426,9 +431,9 @@ data ClassDecl
     | ClsTyDef   SrcLoc Type    Type
             -- ^ default choice for an associated type synonym
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Declarations inside an instance declaration.
@@ -444,9 +449,9 @@ data InstDecl
     | InsInline SrcLoc Bool Activation QName
             -- ^ an INLINE pragma
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | The type of a constructor argument or field, optionally including
@@ -456,9 +461,9 @@ data BangType
      | UnBangedTy Type  -- ^ non-strict component
      | UnpackedTy Type  -- ^ unboxed component, marked with an UNPACK pragma
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | The right hand side of a function or pattern binding.
@@ -467,9 +472,9 @@ data Rhs
      | GuardedRhss  [GuardedRhs]
                 -- ^ guarded right hand side (/gdrhs/)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A guarded right hand side @|@ /stmts/ @=@ /exp/.
@@ -478,9 +483,9 @@ data Rhs
 data GuardedRhs
      = GuardedRhs SrcLoc [Stmt] Exp
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A type qualified with a context.
@@ -500,27 +505,28 @@ data Type
      | TyInfix Type QName Type  -- ^ infix type constructor
      | TyKind  Type Kind        -- ^ type with explicit kind signature
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
--- | Flag denoting whether a tuple is boxed or unboxed.
+{-- | Flag denoting whether a tuple is boxed or unboxed.
 data Boxed = Boxed | Unboxed
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
   deriving (Eq,Ord,Show)
 #endif
+--}
 
 -- | A type variable declaration, optionally with an explicit kind annotation.
 data TyVarBind
     = KindedVar Name Kind   -- ^ variable binding with kind annotation
     | UnkindedVar Name      -- ^ ordinary variable binding
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An explicit kind annotation.
@@ -528,10 +534,12 @@ data Kind
     = KindStar          -- ^ @*@, the kind of types
     | KindBang          -- ^ @!@, the kind of unboxed types
     | KindFn Kind Kind  -- ^ @->@, the kind of a type constructor
+    | KindParen Kind    -- ^ a kind surrounded by parentheses
+    | KindVar Name      -- ^ a kind variable (as of yet unsupported by compilers)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 
@@ -540,9 +548,9 @@ data Kind
 data FunDep
     = FunDep [Name] [Name]
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A context is a set of assertions
@@ -557,9 +565,9 @@ data Asst = ClassA QName [Type]     -- ^ ordinary class assertion
           | IParam IPName Type      -- ^ implicit parameter assertion
           | EqualP Type   Type      -- ^ type equality constraint
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | /literal/
@@ -578,9 +586,9 @@ data Literal
     | PrimChar   Char       -- ^ unboxed character literal
     | PrimString String     -- ^ unboxed string literal
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Haskell expressions.
@@ -601,6 +609,7 @@ data Exp
                                     -- should be an expression.
     | MDo [Stmt]                -- ^ @mdo@-expression
     | Tuple [Exp]               -- ^ tuple expression
+    | TupleSection [Maybe Exp]  -- ^ tuple section expression, e.g. @(,,3)@
     | List [Exp]                -- ^ list expression
     | Paren Exp                 -- ^ parenthesised expression
     | LeftSection Exp QOp       -- ^ left section @(@/exp/ /qop/@)@
@@ -650,9 +659,9 @@ data Exp
     | LeftArrHighApp  Exp Exp   -- ^ higher-order arrow application (from left): /exp/ @-<<@ /exp/
     | RightArrHighApp Exp Exp   -- ^ higher-order arrow application (from right): /exp/ @>>-@ /exp/
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | The name of an xml element or attribute,
@@ -661,17 +670,17 @@ data XName
     = XName String              -- <name ...
     | XDomName String String    -- <dom:name ...
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An xml attribute, which is a name-expression pair.
 data XAttr = XAttr XName Exp
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A template haskell bracket expression.
@@ -681,9 +690,9 @@ data Bracket
     | TypeBracket Type      -- ^ type bracket: @[t| ... |]@
     | DeclBracket [Decl]    -- ^ declaration bracket: @[d| ... |]@
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A template haskell splice expression
@@ -691,9 +700,9 @@ data Splice
     = IdSplice String       -- ^ variable splice: @$var@
     | ParenSplice Exp       -- ^ parenthesised expression splice: @$(/exp/)@
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | The safety of a foreign function call.
@@ -701,9 +710,9 @@ data Safety
     = PlayRisky         -- ^ unsafe
     | PlaySafe Bool     -- ^ safe ('False') or threadsafe ('True')
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | The calling convention of a foreign function call.
@@ -711,9 +720,9 @@ data CallConv
     = StdCall
     | CCall
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A top level options pragma, preceding the module header.
@@ -724,18 +733,19 @@ data OptionPragma
     | OptionsPragma    SrcLoc (Maybe Tool) String
                         -- ^ OPTIONS pragma, possibly qualified with a tool, e.g. OPTIONS_GHC
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
--- | Recognised tools for OPTIONS pragmas.
+{-- | Recognised tools for OPTIONS pragmas.
 data Tool = GHC | HUGS | NHC98 | YHC | HADDOCK | UnknownTool String
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
+-}
 
 -- | Activation clause of a RULES pragma.
 data Activation
@@ -743,18 +753,18 @@ data Activation
     | ActiveFrom  Int
     | ActiveUntil Int
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | The body of a RULES pragma.
 data Rule
     = Rule String Activation (Maybe [RuleVar]) Exp Exp
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Variables used in a RULES pragma, optionally annotated with types
@@ -762,9 +772,9 @@ data RuleVar
     = RuleVar Name
     | TypedRuleVar Name Type
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | Warning text to optionally use in the module header of e.g.
@@ -773,9 +783,9 @@ data WarningText
     = DeprText String
     | WarnText String
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 
@@ -814,17 +824,17 @@ data Pat
     | PBangPat Pat                  -- ^ strict (bang) pattern: @f !x = ...@
 
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An XML attribute in a pattern.
 data PXAttr = PXAttr XName Pat
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A regular pattern operator.
@@ -836,9 +846,9 @@ data RPatOp
     | RPOpt     -- ^ @?@ = 0 or 1
     | RPOptG    -- ^ @?!@ = 0 or 1, greedy
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An entity in a regular pattern.
@@ -852,9 +862,9 @@ data RPat
     | RPParen RPat
     | RPPat Pat
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An /fpat/ in a labeled record pattern.
@@ -863,9 +873,9 @@ data PatField
     | PFieldPun Name            -- ^ record field pun
     | PFieldWildcard            -- ^ record field wildcard
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A statement, representing both a /stmt/ in a @do@-expression,
@@ -881,9 +891,9 @@ data Stmt
     | LetStmt Binds -- ^ local bindings
     | RecStmt [Stmt]    -- ^ a recursive binding group for arrows
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A general /transqual/ in a list comprehension,
@@ -897,9 +907,9 @@ data QualStmt
     | GroupUsing   Exp          -- ^ @then@ @group@ @using@ /exp/
     | GroupByUsing Exp Exp      -- ^ @then@ @group@ @by@ /exp/ @using@ /exp/
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An /fbind/ in a labeled construction or update expression.
@@ -908,18 +918,18 @@ data FieldUpdate
     | FieldPun Name             -- ^ record field pun
     | FieldWildcard             -- ^ record field wildcard
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | An /alt/ alternative in a @case@ expression.
 data Alt
     = Alt SrcLoc Pat GuardedAlts Binds
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | The right-hand sides of a @case@ alternative,
@@ -929,18 +939,18 @@ data GuardedAlts
     = UnGuardedAlt Exp          -- ^ @->@ /exp/
     | GuardedAlts  [GuardedAlt] -- ^ /gdpat/
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -- | A guarded case alternative @|@ /stmts/ @->@ /exp/.
 data GuardedAlt
     = GuardedAlt SrcLoc [Stmt] Exp
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data)
 #else
-  deriving (Eq,Show)
+  deriving (Eq,Ord,Show)
 #endif
 
 -----------------------------------------------------------------------------
